/****************************************************************************
*
*   Copyright (c) 2007 www.societyofrobots.com
*   (please link back if you use this code!)
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License version 2 as
*   published by the Free Software Foundation.
*
*   Alternatively, this software may be distributed under the terms of BSD
*   license.
*
*	Photovore v1, March 10th, 2007
*	Simple case-based method for a robot that chases light.
*
*
****************************************************************************/

//SoR Include
#include "SoR_Utils.h" //includes all the technical stuff

//global variables
int sensor_left=0;//left photoresistor
int sensor_scan=0;//right photoresistor

int left_thresh=0;//threshold value of left sensor
int scan_thresh=0;//threshold value of scanning sensor

int scan_angle=30;//position of scanner, in units of servo command
int max_scan_angle=56;//maximum position scanner can rotate to (57)

int scan_thresh_max=0;//autocalibrate
int scan_thresh_min=255;//autocalibrate

int i=0;

//this function causes scanning servo to center on edge of line
void scan(void)
	{
	//lower (-) goes right
	//higher (+) goes left
	//30 far right, 50 straight, 56 far left (until it jams)

	/*psuedocode
	while line is detected
		scan right while line detected
	while line not detected
		scan left until line detected*/

	sensor_scan=a2dConvert8bit(5);
	
	if (sensor_scan > scan_thresh)//line detected
		{
		if (scan_angle>41) //overflow protection
			scan_angle-=2;//scan right
		}
	else //line not detected
		{
		if (scan_angle<=max_scan_angle) //maximum servo angle
			scan_angle+=2; //scan left
		//else //if scanned all the way, this forces it to start over
		//	scan_angle=30;
		}
		
	//servo scan code
	servo_scan(scan_angle);
	}


//automatically calculates threshold values before run
void autocalibrate(void)
	{
	int i=250;

	scan_angle=42;

	//reset scanner to far right position
	for(i=250; i>0; i--)
		servo_scan(scan_angle);
	
	delay_cycles(3000);//give time for servo to keep up

	//do full scan sweep, slowly moving scanner left
	while(scan_angle <= max_scan_angle)
		{
		sensor_scan=a2dConvert8bit(5);//sensor reading

		//save only highest and lowest value
		if(sensor_scan>scan_thresh_max)
			scan_thresh_max=sensor_scan;
		if(sensor_scan<scan_thresh_min)
			scan_thresh_min=sensor_scan;
			
		//rotate servo to next position
		scan_angle+=1;
		servo_scan(scan_angle);
		servo_scan(scan_angle);

		delay_cycles(6000);//give time for servo to keep up, and photoresistor to adjust
		}
		
	//set average as threshold
	scan_thresh=(long int)(scan_thresh_max + scan_thresh_min)/2;//lower thresh makes it turn left more (harder to see line)
	}

//runs throughout to adapt to line color changes
//deteriorates max and min, but also finds new ones
void sensor_auto_adapt(void)
	{
	sensor_scan=a2dConvert8bit(5);//sensor reading

	//save only highest and lowest value
	if(sensor_scan>scan_thresh_max)
		scan_thresh_max=sensor_scan;
	if(sensor_scan<scan_thresh_min)
		scan_thresh_min=sensor_scan;
		
	//set average as threshold
	scan_thresh=(long int)(scan_thresh_max + scan_thresh_min)/2;

	//deteriorate values
	scan_thresh_max--;
	scan_thresh_min++;
	}

int main(void)
	{
	//LED_on();

	initialize();
	
	delay_cycles(20000);//give time to cover shroud; 21000 = ~1 second

	/*********ADD YOUR CODE BELOW THIS LINE **********/

	//find thresholds
	autocalibrate();

	//LED_off();


	while(1)
		{
		//check for possible line split
		//sensor_left=a2dConvert8bit(5);
	
		//attempts to adapt to changing light conditions - potentially dangerous!
		//sensor_auto_adapt();

//hold_position();

//line following code

		//if(sensor_left<left_thresh && scan_angle>40)//does not see the line and scanner not stretched
			//{
			scan();
		
			//first 2/3rds of Mobot course, use single sensor
			//line on left
			if(scan_angle > 54)
				robot_turn_left();

			//line on right
			else if(scan_angle < 45)
				robot_turn_right();

			//line is centered
			else
				robot_go_straight();
			//}


///////////trapped in a box code///////////
/*if (a2dConvert8bit(5) > scan_thresh)
	{
	for(i=0;i<15;i++)
		{
		robot_go_reverse();
		delay_cycles(1000);
		}
	for(i=0;i<30;i++)
		{
		robot_turn_left();
		delay_cycles(1000);
		}
	}
robot_go_straight();*/
///////////////////////////////

		//rprintf("Initialization Complete\r\n");
		
		//output message to serial (use hyperterminal)
		//print("Hello, World! Read My Analog: %u\r\n", sensor_0);

		delay_cycles(400);//a small delay to prevent crazy oscillations
		}
	/*********ADD YOUR CODE ABOVE THIS LINE **********/

	return 0;
	}


/*********************COMMAND LIST*************************

delay_cycles(cycles);
Delays - you can make your robot wait for a certain amount of time with this function.
Put the number of computational cycles to delay in the ().
23 cycles is about .992 milliseconds
to calculate: 23/.992*(time in milliseconds to delay) = cycles
Check servo datasheet where it says: 'Direction: Clockwise/Pulse Traveling 1500 to 1900usec'


servo_left(speed); and servo_right(speed);
Commands your servos to rotate at a certain speed.
Vary speed (which represents a delay in cycles) from 20 to 50.
Left is for port D0 and right is for port D1.


LED_on(); and LED_off();
Turns on and off your LED. The LED is on port D4.
By bringing port D4 low, you are turning on the LED.


variable=a2dConvert8bit(pin);
Reads analog pin. For example, set 'pin' to 5 to read PC5.
'variable' will store the value.

***********************************************************/
